------------------------------------------------------------------------------
-- GridStatusGermination by Slaren
------------------------------------------------------------------------------
GridStatusGermination = Grid:GetModule("GridStatus"):NewModule("GridStatusGermination")

local REJUV_SPELLID = 774
local GERM_SPELLID = 155777

-- locals
local GridRoster = Grid:GetModule("GridRoster")
local GetSpellInfo = GetSpellInfo
local UnitAura = UnitAura
local UnitGUID = UnitGUID
local tostring = tostring

local settings
local rejuv_name
local germ_name
local rejuv_icon


GridStatusGermination.defaultDB = {
	alert_germination = {
		enable = true,
		color = { r = 1, g = 1, b = 1, a = 1, ignore = true },
		priority = 99,
		range = false,
	}
}

local myoptions = {
}

function GridStatusGermination:OnInitialize()
	self.super.OnInitialize(self)
	
	local _
	rejuv_name, _, rejuv_icon = GetSpellInfo(REJUV_SPELLID)
	germ_name = GetSpellInfo(GERM_SPELLID)
	
	self:RegisterStatus("alert_germination", "Germination", myoptions, true)

	settings = self.db.profile.alert_germination
end

function GridStatusGermination:OnStatusEnable(status)
	if status == "alert_germination" then
		self:RegisterEvent("UNIT_AURA", "ScanUnit")
		self:RegisterEvent("Grid_UnitJoined")
		-- self:ScheduleRepeatingEvent("GridStatusGermination:UpdateAllUnits", self.UpdateAllUnits, 0.5, self)
		self:UpdateAllUnits()
	end
end

function GridStatusGermination:OnStatusDisable(status)
	if status == "alert_germination" then
		self:UnregisterEvent("UNIT_AURA")
		self:UnregisterEvent("Grid_UnitJoined")

		--self:CancelScheduledEvent("GridStatusGermination:UpdateAllUnits")
		self.core:SendStatusLostAllUnits("alert_germination")
	end
end

function GridStatusGermination:Grid_UnitJoined(guid, unitid)
	self:ScanUnit("Grid_UnitJoined", unitid, guid)
end

function GridStatusGermination:UpdateAllUnits()
	for guid, unitid in GridRoster:IterateRoster() do
		self:ScanUnit("UpdateAllUnits", unitid, guid)
	end
end

function GridStatusGermination:ScanUnit(event, unitid, unitguid)
	unitguid = unitguid or UnitGUID(unitid)
	if not GridRoster:IsGUIDInRaid(unitguid) then
		return
	end

	local rejuv_name, _, _, _, _, rejuv_duration, rejuv_expiration = UnitAura(unitid, rejuv_name, nil, "HELPFUL|PLAYER")
	local germ_name, _, _, _, _, germ_duration, germ_expiration = UnitAura(unitid, germ_name, nil, "HELPFUL|PLAYER")
	
	if rejuv_name or germ_name then
		local count = (rejuv_name and 1 or 0) + (germ_name and 1 or 0)
		local start
		local duration

		if not germ_expiration or (rejuv_expiration and rejuv_expiration < germ_expiration) then
			start = rejuv_expiration - rejuv_duration
			duration = rejuv_duration
		else
			start = germ_expiration - germ_duration
			duration = germ_duration
		end

		self.core:SendStatusGained(unitguid, 
					"alert_germination",
					settings.priority,
					(settings.range and 40),
					settings.color,
					tostring(count),
					0,							-- value
					nil,						-- maxValue
					rejuv_icon,					-- icon
					start,						-- start
					duration,					-- duration
					count)						-- stack
	else
		self.core:SendStatusLost(unitguid, "alert_germination")
	end
end
